@@ -186,6 +186,8 @@ list_column_to_sfc = function(x) {
 #' @param sf_column_name character; name of the active list-column with simple feature geometries; in case 
 #' there are more than one and \code{sf_column_name} is not given, the first one is taken.
 #' @details \code{agr}, attribute-geometry-relationship, specifies for each non-geometry attribute column how it relates to the geometry, and can have one of following values: "constant", "aggregate", "identity". "constant" is used for attributes that are constant throughout the geometry (e.g. land use), "aggregate" where the attribute is an aggregate value over the geometry (e.g. population density or population count), "identity" when the attributes uniquely identifies the geometry of particular "thing", such as a building ID or a city name. The default value, \code{NA_agr_}, implies we don't know.  
+#' 
+#' When confronted with a data.frame-like object, `st_sf` will try to find a geometry column of class `sfc`, and otherwise try to convert list-columns when available into a geometry column, using \link{st_as_sfc}.
 #' @examples
 #' g = st_sfc(st_point(1:2))
 #' st_sf(a=3,g)
@@ -424,3 +426,9 @@ st_as_sfc.list = function(x, ..., crs = NA_crs_) {
 			st_as_sfc(unlist(x), ...) # wkt
 	}
 }
+
+#' @name st_as_sfc
+#' @export
+st_as_sfc.blob = function(x, ...) {
+	st_as_sfc.list(x, ...)
+}
