@@ -70,18 +70,18 @@ void handle_error(OGRErr err) {
 		switch (err) {
 			case OGRERR_NOT_ENOUGH_DATA:
 				Rcpp::Rcout << "OGR: Not enough data " << std::endl; // #nocov
-				break;
+				break; // #nocov
 			case OGRERR_UNSUPPORTED_GEOMETRY_TYPE:
 				Rcpp::Rcout << "OGR: Unsupported geometry type" << std::endl;
 				break;
-			case OGRERR_CORRUPT_DATA:                   		 	// #nocov
-				Rcpp::Rcout << "OGR: Corrupt data" << std::endl;    // #nocov
-				break;
+			case OGRERR_CORRUPT_DATA:
+				Rcpp::Rcout << "OGR: Corrupt data" << std::endl;     // #nocov
+				break; // #nocov
 			case OGRERR_FAILURE:
 				Rcpp::Rcout << "OGR: index invalid?" << std::endl;    // #nocov
-				break;
+				break; // #nocov
 			default:
-				Rcpp::Rcout << "Error code: " << err << std::endl;  // #nocov
+				Rcpp::Rcout << "Error code: " << err << std::endl;    // #nocov
 		}
 		throw std::range_error("OGR error");
 	}
